@@ -4,19 +4,20 @@
 #' 
 #' @param mdcnev_settings List of settings for the MDCEV model. It must include the following.
 #'                       \itemize{
-#'                         \item V: Named list. Utilities of the alternatives. Names of elements must match those in argument 'alternatives'.
-#'                         \item alternatives: Character vector. Names of alternatives, elements must match the names in list 'V'.
-#'                         \item alpha: Named list. Alpha parameters for each alternative, including for the outside good. As many elements as alternatives.
-#'                         \item gamma: Named list. Gamma parameters for each alternative, including for the outside good. As many elements as alternatives.
-#'                         \item mdcnevNests: Named list. Lambda parameters for each nest. Elements must be named with the nest name. The lambda at the root is fixed to 1, and therefore must be no be defined. The value of the estimated mdcnevNests parameters should be between 0 and 1 to ensure consistency with random utility maximization.
-#'                         \item mdcnevStructure: Numeric matrix. One row per nest and one column per alternative. Each element of the matrix is 1 if an alternative belongs to the corresponding nest.
-#'                         \item cost: Named list of numeric vectors. Price of each alternative. One element per alternative, each one as long as the number of observations or a scalar. Names must match those in \code{alternatives}.
-#'                         \item avail: Named list. Availabilities of alternatives, one element per alternative. Names of elements must match those in argument 'alternatives'. Value for each element can be 1 (scalar if always available) or a vector with values 0 or 1 for each observation. If all alternatives are always available, then user can just omit this argument.
-#'                         \item continuousChoice: Named list of numeric vectors. Amount of consumption of each alternative. One element per alternative, as long as the number of observations or a scalar. Names must match those in \code{alternatives}.
-#'                         \item budget: Numeric vector. Budget for each observation.
-#'                         \item minConsumption: Named list of scalars or numeric vectors. Minimum consumption of the alternatives, if consumed. As many elements as alternatives. Names must match those in \code{alternatives}.
-#'                         \item outside: Character. Alternative name for the outside good. Default is "outside"
-#'                         \item rows: Boolean vector. Consideration of rows in the likelihood calculation, FALSE to exclude. Length equal to the number of observations (nObs). Default is \code{"all"}, equivalent to \code{rep(TRUE, nObs)}.
+#'                         \item \strong{\code{V}}: Named list. Utilities of the alternatives. Names of elements must match those in argument 'alternatives'.
+#'                         \item \strong{\code{alternatives}}: Character vector. Names of alternatives, elements must match the names in list 'V'.
+#'                         \item \strong{\code{alpha}}: Named list. Alpha parameters for each alternative, including for the outside good. As many elements as alternatives.
+#'                         \item \strong{\code{gamma}}: Named list. Gamma parameters for each alternative, including for the outside good. As many elements as alternatives.
+#'                         \item \strong{\code{mdcnevNests}}: Named list. Lambda parameters for each nest. Elements must be named with the nest name. The lambda at the root is fixed to 1, and therefore must be no be defined. The value of the estimated mdcnevNests parameters should be between 0 and 1 to ensure consistency with random utility maximization.
+#'                         \item \strong{\code{mdcnevStructure}}: Numeric matrix. One row per nest and one column per alternative. Each element of the matrix is 1 if an alternative belongs to the corresponding nest.
+#'                         \item \strong{\code{cost}}: Named list of numeric vectors. Price of each alternative. One element per alternative, each one as long as the number of observations or a scalar. Names must match those in \code{alternatives}.
+#'                         \item \strong{\code{avail}}: Named list. Availabilities of alternatives, one element per alternative. Names of elements must match those in argument 'alternatives'. Value for each element can be 1 (scalar if always available) or a vector with values 0 or 1 for each observation. If all alternatives are always available, then user can just omit this argument.
+#'                         \item \strong{\code{continuousChoice}}: Named list of numeric vectors. Amount of consumption of each alternative. One element per alternative, as long as the number of observations or a scalar. Names must match those in \code{alternatives}.
+#'                         \item \strong{\code{budget}}: Numeric vector. Budget for each observation.
+#'                         \item \strong{\code{minConsumption}}: Named list of scalars or numeric vectors. Minimum consumption of the alternatives, if consumed. As many elements as alternatives. Names must match those in \code{alternatives}.
+#'                         \item \strong{\code{outside}}: Character. Alternative name for the outside good. Default is "outside"
+#'                         \item \strong{\code{rows}}: Boolean vector. Consideration of rows in the likelihood calculation, FALSE to exclude. Length equal to the number of observations (nObs). Default is \code{"all"}, equivalent to \code{rep(TRUE, nObs)}.
+#'                         \item \strong{\code{componentName}}: Character. Name given to model component.
 #'                       }
 #' @param functionality Character. Can take different values depending on desired output.
 #'                      \itemize{
@@ -30,28 +31,31 @@
 #'                      }
 #' @return The returned object depends on the value of argument \code{functionality} as follows.
 #'         \itemize{
-#'           \item "estimate": vector/matrix/array. Returns the probabilities for the chosen alternative for each observation.
-#'           \item "prediction": A matrix with one row per observation, and means and s.d. of predicted consumptions.
-#'           \item "validate": Boolean. Returns TRUE if all tests are passed.
-#'           \item "zero_LL": Not applicable.
-#'           \item "conditionals": Same as "prediction".
-#'           \item "output": Same as "estimate" but also writes summary of choices into temporary file (later read by \code{apollo_modelOutput}).
-#'           \item "raw": Same as "prediction".
+#'           \item \strong{\code{"estimate"}}: vector/matrix/array. Returns the probabilities for the observed consumption for each observation.
+#'           \item \strong{\code{"prediction"}}: A matrix with one row per observation, and columns indicating means and s.d. of continuous and discrete predicted consumptions.
+#'           \item \strong{\code{"validate"}}: Same as \code{"estimate"}, but it also runs a set of tests to validate the function inputs.
+#'           \item \strong{\code{"zero_LL"}}: Not implemented. Returns a vector of NA with as many elements as observations.
+#'           \item \strong{\code{"conditionals"}}: Same as \code{"estimate"}
+#'           \item \strong{\code{"output"}}: Same as \code{"estimate"} but also writes summary of input data to internal Apollo log.
+#'           \item \strong{\code{"raw"}}: Same as \code{"estimate"}
 #'         }
 #' @export
 #' @importFrom mnormt rmnorm
 #' @importFrom stats setNames
 apollo_mdcnev <- function(mdcnev_settings,functionality){
-  if(is.null(mdcnev_settings[["alternatives"]])) stop("The mdcnev_settings list needs to include an object called \"alternatives\"!")
-  if(is.null(mdcnev_settings[["avail"]])) stop("The mdcnev_settings list needs to include an object called \"avail\"!")
-  if(is.null(mdcnev_settings[["continuousChoice"]])) stop("The mdcnev_settings list needs to include an object called \"continuousChoice\"!")
-  if(is.null(mdcnev_settings[["V"]])) stop("The mdcnev_settings list needs to include an object called \"V\"!")
-  if(is.null(mdcnev_settings[["alpha"]])) stop("The mdcnev_settings list needs to include an object called \"alpha\"!")
-  if(is.null(mdcnev_settings[["gamma"]])) stop("The mdcnev_settings list needs to include an object called \"gamma\"!")
-  if(is.null(mdcnev_settings[["mdcnevNests"]])) stop("The mdcnev_settings list needs to include an object called \"mdcnevNests\"!")
-  if(is.null(mdcnev_settings[["mdcnevStructure"]])) stop("The mdcnev_settings list needs to include an object called \"mdcnevStructure\"!")
-  if(is.null(mdcnev_settings[["cost"]])) stop("The mdcnev_settings list needs to include an object called \"cost\"!")
-  if(is.null(mdcnev_settings[["budget"]])) stop("The mdcnev_settings list needs to include an object called \"budget\"!")
+  if(is.null(mdcnev_settings[["componentName"]])) mdcnev_settings[["componentName"]]="MDCNEV"
+  componentName     = mdcnev_settings[["componentName"]]
+  
+  if(is.null(mdcnev_settings[["alternatives"]])) stop("The mdcnev_settings list for model component \"",componentName,"\" needs to include an object called \"alternatives\"!")
+  if(is.null(mdcnev_settings[["avail"]])) stop("The mdcnev_settings list for model component \"",componentName,"\" needs to include an object called \"avail\"!")
+  if(is.null(mdcnev_settings[["continuousChoice"]])) stop("The mdcnev_settings list for model component \"",componentName,"\" needs to include an object called \"continuousChoice\"!")
+  if(is.null(mdcnev_settings[["V"]])) stop("The mdcnev_settings list for model component \"",componentName,"\" needs to include an object called \"V\"!")
+  if(is.null(mdcnev_settings[["alpha"]])) stop("The mdcnev_settings list for model component \"",componentName,"\" needs to include an object called \"alpha\"!")
+  if(is.null(mdcnev_settings[["gamma"]])) stop("The mdcnev_settings list for model component \"",componentName,"\" needs to include an object called \"gamma\"!")
+  if(is.null(mdcnev_settings[["mdcnevNests"]])) stop("The mdcnev_settings list for model component \"",componentName,"\" needs to include an object called \"mdcnevNests\"!")
+  if(is.null(mdcnev_settings[["mdcnevStructure"]])) stop("The mdcnev_settings list for model component \"",componentName,"\" needs to include an object called \"mdcnevStructure\"!")
+  if(is.null(mdcnev_settings[["cost"]])) stop("The mdcnev_settings list for model component \"",componentName,"\" needs to include an object called \"cost\"!")
+  if(is.null(mdcnev_settings[["budget"]])) stop("The mdcnev_settings list for model component \"",componentName,"\" needs to include an object called \"budget\"!")
   if(is.null(mdcnev_settings[["minConsumption"]])) mdcnev_settings[["minConsumption"]]=NA
   if(is.null(mdcnev_settings[["rows"]])) mdcnev_settings[["rows"]]="all"
   if(is.null(mdcnev_settings[["outside"]])) mdcnev_settings[["outside"]]=NA ### new
@@ -70,6 +74,7 @@ apollo_mdcnev <- function(mdcnev_settings,functionality){
   minConsumption    = mdcnev_settings[["minConsumption"]]
   rows              = mdcnev_settings[["rows"]]
   outsideName       = mdcnev_settings[["outside"]]   ### new
+  
   if(anyNA(outsideName) && "outside" %in% alternatives) outsideName = "outside"  
   if(anyNA(outsideName)) stop("No outside good provided for MDCNEV model!")      
   if(is.null(gamma[[outsideName]])) gamma[[outsideName]]=1
@@ -106,10 +111,10 @@ apollo_mdcnev <- function(mdcnev_settings,functionality){
 
   if(functionality=="validate"){
 
-    if(!is.null(gamma[["outside"]])){
-      cat("\nWarning: You provided a gamma for the outside good, which will be ignored in the model\n")
-      gamma[["outside"]]=NULL
-    }
+    #if(!is.null(gamma[["outside"]])){
+    #  cat("\nWarning: You provided a gamma for the outside good,\n         which will be ignored in the model.\n")
+    #  gamma[["outside"]]=NULL
+    #}
 
     # Store useful values
     nObs  <- length(continuousChoice[[1]])
@@ -216,7 +221,7 @@ apollo_mdcnev <- function(mdcnev_settings,functionality){
       
       # Set utility of unavailable alternatives and excluded rows to 0 to avoid numerical issues
       V <- mapply(function(v,a) apollo_setRows(v, !a | !rows, 0), V, avail, SIMPLIFY=FALSE)
-      if(any(sapply(V, anyNA))) warning("At least one utility contains one or more NA values")
+      if(any(sapply(V, anyNA))) cat("\nAt least one utility contains one or more NA values")
 
       # checks that are specific to cnlStructure component
       if(nrow(mdcnevStructure)!=length(mdcnevNests)) stop("Tree structure needs one row per nest!")
@@ -256,7 +261,10 @@ apollo_mdcnev <- function(mdcnev_settings,functionality){
       apollo_addLog("Overview of choices for MDCEV model component:", content, apolloLog)
 
     }
-    return(TRUE)
+    testL=apollo_mdcnev(mdcnev_settings, functionality="estimate")
+    if(all(testL==0)) stop("\nAll observations have zero probability at starting value for model component \"",componentName,"\"")
+    if(any(testL==0)) cat("\nSome observations have zero probability at starting value for model component \"",componentName,"\"")
+    return(invisible(testL))
   }
   #if(!is.null(gamma[["outside"]])) gamma[["outside"]]=NULL
   
@@ -593,7 +601,10 @@ apollo_mdcnev <- function(mdcnev_settings,functionality){
     content[[length(content) + 1]] <- mdcnevStructure
     apolloLog <- tryCatch(get("apollo_inputs", parent.frame(), inherits=TRUE )$apolloLog, error=function(e) return(NA))
     apollo_addLog("Overview of choices for MDCEV model component:", content, apolloLog)
-
+    apollo_addLog(title   = paste0("Final structure for MDCNEV model component \"",componentName,"\":"), 
+                  content = mdcnevStructure, apolloLog, 
+                  book    = 2)
+    apollo_reportModelTypeLog(modelType="MDCNEV", apolloLog)
 
     return(P)
   }
@@ -766,7 +777,7 @@ apollo_mdcnev <- function(mdcnev_settings,functionality){
 
     if(apollo_control$mixing==FALSE){
 
-      cat("\nNow producing forecasts from MDCNEV model component.")
+      cat("\nNow producing forecasts for model component \"",componentName,"\".",sep="")
       cat("\nA matrix with one row per observation and the following columns will be returned:")
       cat("\n1. Means of predicted continuous consumptions (one column per product)")
       cat("\n2. Std err of predicted continuous consumptions (one column per product)")
