@@ -4,31 +4,32 @@
 #'
 #' @param mnl_settings List of inputs of the MNL model. It should contain the following.
 #'                     \itemize{
-#'                       \item alternatives: Named numeric vector. Names of alternatives and their corresponding value in \code{choiceVar}.
-#'                       \item avail: Named list of numeric vectors or scalars. Availabilities of alternatives, one element per alternative. Names of elements must match those in \code{alternatives}. Values can be 0 or 1.
-#'                       \item choiceVar: Numeric vector. Contains choices for all observations. It will usually be a column from the database. Values are defined in \code{alternatives}.
-#'                       \item V: Named list of deterministic utilities . Utilities of the alternatives. Names of elements must match those in \code{alternatives.}
-#'                       \item rows: Boolean vector. Consideration of rows in the likelihood calculation, FALSE to exclude. Length equal to the number of observations (nObs). Default is \code{"all"}, equivalent to \code{rep(TRUE, nObs)}.
+#'                       \item \strong{\code{alternatives}}: Named numeric vector. Names of alternatives and their corresponding value in \code{choiceVar}.
+#'                       \item \strong{\code{avail}}: Named list of numeric vectors or scalars. Availabilities of alternatives, one element per alternative. Names of elements must match those in \code{alternatives}. Values can be 0 or 1.
+#'                       \item \strong{\code{choiceVar}}: Numeric vector. Contains choices for all observations. It will usually be a column from the database. Values are defined in \code{alternatives}.
+#'                       \item \strong{\code{V}}: Named list of deterministic utilities . Utilities of the alternatives. Names of elements must match those in \code{alternatives.}
+#'                       \item \strong{\code{rows}}: Boolean vector. Consideration of rows in the likelihood calculation, FALSE to exclude. Length equal to the number of observations (nObs). Default is \code{"all"}, equivalent to \code{rep(TRUE, nObs)}.
+#'                       \item \strong{\code{componentName}}: Character. Name given to model component.
 #'                     }
 #' @param functionality Character. Can take different values depending on desired output.
 #'                      \itemize{
-#'                        \item "estimate": Used for model estimation.
-#'                        \item "prediction": Used for model predictions.
-#'                        \item "validate": Used for validating input.
-#'                        \item "zero_LL": Used for calculating null likelihood.
-#'                        \item "conditionals": Used for calculating conditionals.
-#'                        \item "output": Used for preparing output after model estimation.
-#'                        \item "raw": Used for debugging.
+#'                        \item \code{"estimate"}: Used for model estimation.
+#'                        \item \code{"prediction"}: Used for model predictions.
+#'                        \item \code{"validate"}: Used for validating input.
+#'                        \item \code{"zero_LL"}: Used for calculating null likelihood.
+#'                        \item \code{"conditionals"}: Used for calculating conditionals.
+#'                        \item \code{"output"}: Used for preparing output after model estimation.
+#'                        \item \code{"raw"}: Used for debugging.
 #'                      }
 #' @return The returned object depends on the value of argument \code{functionality} as follows.
 #'         \itemize{
-#'           \item "estimate": vector/matrix/array. Returns the probabilities for the chosen alternative for each observation.
-#'           \item "prediction": List of vectors/matrices/arrays. Returns a list with the probabilities for all alternatives, with an extra element for the chosen alternative probability.
-#'           \item "validate": Boolean. Returns TRUE if all tests are passed.
-#'           \item "zero_LL": vector/matrix/array. Returns the probability of the chosen alternative when all parameters are zero.
-#'           \item "conditionals": Same as "prediction".
-#'           \item "output": Same as "estimate" but also writes summary of choices into temporary file (later read by \code{apollo_modelOutput}).
-#'           \item "raw": Same as "prediction".
+#'           \item \strong{\code{"estimate"}}: vector/matrix/array. Returns the probabilities for the chosen alternative for each observation.
+#'           \item \strong{\code{"prediction"}}: List of vectors/matrices/arrays. Returns a list with the probabilities for all alternatives, with an extra element for the probability of the chosen alternative.
+#'           \item \strong{\code{"validate"}}: Same as \code{"estimate"}, but it also runs a set of tests to validate the function inputs.
+#'           \item \strong{\code{"zero_LL"}}: vector/matrix/array. Returns the probability of the chosen alternative when all parameters are zero.
+#'           \item \strong{\code{"conditionals"}}: Same as \code{"estimate"}
+#'           \item \strong{\code{"output"}}: Same as \code{"estimate"} but also writes summary of input data to internal Apollo log.
+#'           \item \strong{\code{"raw"}}: Same as \code{"prediction"}
 #'         }
 #' @examples
 #' ### Load data
@@ -62,10 +63,13 @@
 #' apollo_mnl(mnl_settings, functionality="estimate")
 #' @export
 apollo_mnl <- function(mnl_settings, functionality){
-  if(is.null(mnl_settings[["alternatives"]])) stop("The mnl_settings list needs to include an object called \"alternatives\"!")
-  if(is.null(mnl_settings[["avail"]])) stop("The mnl_settings list needs to include an object called \"avail\"!")
-  if(is.null(mnl_settings[["choiceVar"]])) stop("The mnl_settings list needs to include an object called \"choiceVar\"!")
-  if(is.null(mnl_settings[["V"]])) stop("The mnl_settings list needs to include an object called \"V\"!")
+  if(is.null(mnl_settings[["componentName"]])) mnl_settings[["componentName"]]="MNL"
+  componentName=mnl_settings[["componentName"]]
+  
+  if(is.null(mnl_settings[["alternatives"]])) stop("The mnl_settings list for model component \"",componentName,"\" needs to include an object called \"alternatives\"!")
+  if(is.null(mnl_settings[["avail"]])) stop("The mnl_settings list for model component \"",componentName,"\" needs to include an object called \"avail\"!")
+  if(is.null(mnl_settings[["choiceVar"]])) stop("The mnl_settings list for model component \"",componentName,"\" needs to include an object called \"choiceVar\"!")
+  if(is.null(mnl_settings[["V"]])) stop("The mnl_settings list for model component \"",componentName,"\" needs to include an object called \"V\"!")
   if(is.null(mnl_settings[["rows"]])) mnl_settings[["rows"]]="all"
   
   ### Useful values
@@ -85,32 +89,33 @@ apollo_mnl <- function(mnl_settings, functionality){
                      return(max(lenV, lenA, lenC))
                    })
   
+  
   ### Format checks
   # alternatives
   test <- is.vector(alternatives) & !is.null(names(alternatives))
-  if(!test) stop("The \"alternatives\" argument needs to be a named vector")
+  if(!test) stop("The \"alternatives\" argument for model component \"",componentName,"\" needs to be a named vector")
   # avail
   test <- is.list(avail) || (length(avail)==1 && avail==1)
-  if(!test) stop("The \"avail\" argument needs to be a list or set to 1")
+  if(!test) stop("The \"avail\" argument for model component \"",componentName,"\" needs to be a list or set to 1")
   if(is.list(avail)){
     lenA <- sapply(avail, function(v) ifelse(is.array(v), dim(v)[1], length(v)) )
     test <- all(lenA==nObs | lenA==1)
-    if(!test) stop("All entries in \"avail\" need to be a scalar or a vector with one entry per observation in the \"database\"")
+    if(!test) stop("All entries in \"avail\" for model component \"",componentName,"\" need to be a scalar or a vector with one entry per observation in the \"database\"")
   }
   # choiceVar
   test <- is.vector(choiceVar) && (length(choiceVar)==nObs || length(choiceVar)==1)
-  if(!test) stop("The \"choiceVar\" argument needs to be a scalar or a vector with one entry per observation in the \"database\"")
+  if(!test) stop("The \"choiceVar\" argument for model component \"",componentName,"\" needs to be a scalar or a vector with one entry per observation in the \"database\"")
   # V
-  if(!is.list(V)) stop("The \"V\" argument needs to be a list")
+  if(!is.list(V)) stop("The \"V\" argument for model component \"",componentName,"\" needs to be a list")
   lenV <- sapply(V, function(v) ifelse(is.array(v), dim(v)[1], length(v)) )
   test <- all(lenV==nObs | lenV==1)
-  if(!test) stop("Each element of \"V\" needs to be a scalar or a vector with one entry per observation in the \"database\"")  
+  if(!test) stop("Each element of \"V\" for model component \"",componentName,"\" needs to be a scalar or a vector/matrix/cube with one row per observation in the \"database\"")  
   # rows
   test <- is.vector(rows) && ( (is.logical(rows) && length(rows)==nObs) || (length(rows)==1 && rows=="all") )
-  if(!test) stop("The \"rows\" argument needs to be \"all\" or a vector of boolean statements with one entry per observation in the \"database\"")
+  if(!test) stop("The \"rows\" argument for model component \"",componentName,"\" needs to be \"all\" or a vector of boolean statements with one entry per observation in the \"database\"")
   # functionality
   test <- functionality %in% c("estimate","prediction","validate","zero_LL","conditionals","output","raw")
-  if(!test) stop("Non-permissable setting for \"functionality\"")
+  if(!test) stop("Non-permissable setting for \"functionality\" for model component \"",componentName,"\"")
   
   ### Expand availabilities if =1
   avail_set <- FALSE
@@ -150,33 +155,33 @@ apollo_mnl <- function(mnl_settings, functionality){
     #if(length(rows)!=length(choiceVar)) stop("The argument \"rows\" needs to either be \"all\" or a vector of length equal to the number of the rows in the data!")
     
     # Check that alternatives are named in altcodes and V
-    if(is.null(altnames) || is.null(altcodes) || is.null(names(V))) stop("Alternatives must be named, both in 'alternatives' and 'V'.")
+    if(is.null(altnames) || is.null(altcodes) || is.null(names(V))) stop("Alternatives for model component \"",componentName,"\" must be named, both in 'alternatives' and 'V'.")
     
     if(apollo_control$noValidation==FALSE){
       # Check that there are at least two alternatives
-      if(nAlt<2) stop("MNL requires at least two alternatives")
+      if(nAlt<2) stop("Model component \"",componentName,"\"  requires at least two alternatives")
       
       # Check that choice vector is not empty
-      if(length(choiceVar)==0) stop("No choices to model")
-      if(nObs==0) stop("No utilities defined")
+      if(length(choiceVar)==0) stop("Choice vector is empty for model component \"",componentName,"\"")
+      if(nObs==0) stop("No data for model component \"",componentName,"\"")
       
       choiceLabs <- unique(choiceVar)
-      if(!all(altnames %in% names(V))) stop("Alternatives names in \"alternatives\" do not match those in \"V\".")
-      if(!all(altnames %in% names(avail))) stop("Alternatives names in \"alternatives\" do not match those in \"avail\".")
+      if(!all(altnames %in% names(V))) stop("The names of the alternatives for model component \"",componentName,"\" do not match those in \"V\".")
+      if(!all(altnames %in% names(avail))) stop("The names of the alternatives for model component \"",componentName,"\" do not match those in \"avail\".")
       
       # Check that there are no values in the choice column for undefined alternatives
-      if(!all(choiceLabs %in% altcodes)) stop('A value in "choiceVar" column is not included in "alternatives".')
+      if(!all(choiceLabs %in% altcodes)) stop("The data contains values for \"choiceVar\" for model component \"",componentName,"\" that are not included in \"alternatives\".")
       
       # check that nothing unavailable is chosen
-      for(j in 1:nAlt) if(any(choiceVar==altcodes[j] & avail[[j]]==0)) stop("Alternative ",altnames[j]," chosen despite being listed as unavailable\n")
+      for(j in 1:nAlt) if(any(choiceVar==altcodes[j] & avail[[j]]==0)) stop("The data contains cases where alternative ",altnames[j]," is chosen for model component \"",componentName,"\" despite being listed as unavailable\n")
       
       # check that all availabilities are either 0 or 1
-      for(i in 1:length(avail)) if( !all(unique(avail[[i]]) %in% 0:1) ) stop("Some availability values are not 0 or 1.")
+      for(i in 1:length(avail)) if( !all(unique(avail[[i]]) %in% 0:1) ) stop("Some availability values for model component \"",componentName,"\" are not 0 or 1.")
       
       # Check that no available alternative has utility = NA
       # Requires setting non available alternatives utility to 0 first
       V <- mapply(function(v,a) apollo_setRows(v, !a, 0), V, avail, SIMPLIFY=FALSE)
-      if(any(sapply(V, anyNA))) warning("At least one utility contains one or more NA values at parameter starting values")
+      if(any(sapply(V, anyNA))) cat("\nAt least one utility for model component \"",componentName,"\" contains one or more NA values at parameter starting values")
     }
     
     if(apollo_control$noDiagnostics==FALSE){
@@ -196,13 +201,16 @@ apollo_mnl <- function(mnl_settings, functionality){
       content <- list(round(choicematrix,2),
                       ifelse(any(choicematrix[4,]==0), "Warning: some alternatives are never chosen in your data!", ""),
                       ifelse(any(choicematrix[4,]==1), "Warning: some alternatives are always chosen when available!", ""))
-      if(avail_set==TRUE) content[[length(content)+1]] <- paste0("Warning: Availability not provided to 'apollo_mnl' (or some elements are NA).",
+      if(avail_set==TRUE) content[[length(content)+1]] <- paste0("Warning: Availability not provided (or some elements are NA).",
                                                                  "\n", paste0(rep(" ",9),collapse=""),"Full availability assumed.")
       apolloLog <- tryCatch(get("apollo_inputs", parent.frame(), inherits=TRUE )$apolloLog, error=function(e) return(NA))
-      apollo_addLog("Overview of choices for MNL model component:", content, apolloLog)
+      apollo_addLog(paste0("Overview of choices for model component \"",componentName,"\""), content, apolloLog)
     }
     
-    return(invisible(TRUE))
+    testL=apollo_mnl(mnl_settings, functionality="estimate")
+    if(all(testL==0)) stop("\nAll observations have zero probability at starting value for model component \"",componentName,"\"")
+    if(any(testL==0)) cat("\nSome observations have zero probability at starting value for model component \"",componentName,"\"")
+    return(invisible(testL))
   }
   
   # ############################## #
@@ -249,20 +257,20 @@ apollo_mnl <- function(mnl_settings, functionality){
     V = lapply(X=V, FUN=function(v) v-chosenV)
     
     # exponentiate utilities
-    eV = lapply(X=V, FUN=exp)
+    V = lapply(X=V, FUN=exp)
     
-    # consider availabilities (it assumes eV and avail are in the same order)
-    eV <- mapply('*', eV, avail, SIMPLIFY = FALSE)
+    # consider availabilities (it assumes V and avail are in the same order)
+    V <- mapply('*', V, avail, SIMPLIFY = FALSE)
     
     # calculate the denominator of the logit probability expression
     # now simply the addition of all exponentiated utilities
-    denom = Reduce('+',eV)
+    denom = Reduce('+',V)
     
     # calculate probabilities
     # if probabilities for all alternatives are requested, then P is a list
     # if only the probability of the chosen alternative is requested, then P is vector or a 3-dim array
     if((functionality=="prediction")|(functionality=="raw")){
-      P <- lapply(eV, function(ev) ev/denom)
+      P <- lapply(V, function(ev) ev/denom)
       if(!choiceNA) P[["chosen"]]= chosenAvail/denom
       if(any(!rows)) P <- lapply(P, apollo_insertRows, r=rows, val=NA) # insert excluded rows with value 1
     } else {
@@ -294,12 +302,14 @@ apollo_mnl <- function(mnl_settings, functionality){
     choicematrix[4,!is.finite(choicematrix[4,])] <- 0
     
     content <- list(round(choicematrix,2),
-                    ifelse(any(choicematrix[4,]==0), "Warning: some alternatives are never chosen in your data!", ""),
-                    ifelse(any(choicematrix[4,]==1), "Warning: some alternatives are always chosen when available!", ""))
-    if(avail_set==TRUE) content[[length(content)+1]] <- paste0("Warning: Availability not provided to 'apollo_mnl' (or some elements are NA).",
-                                                               "\n", paste0(rep(" ",9),collapse=""),"Full availability assumed.")
+                    ifelse(any(choicematrix[4,]==0), "WARNING: some alternatives are never chosen in your data!", ""),
+                    ifelse(any(choicematrix[4,]==1), "WARNING: some alternatives are always chosen when available!", ""))
+    if(avail_set==TRUE) content[[length(content)+1]] <- paste0("WARNING: Availability not provided to 'apollo_mnl' (or some elements are NA).",
+                                                               "\n         Full availability assumed.")
     apolloLog <- tryCatch(get("apollo_inputs", parent.frame(), inherits=TRUE )$apolloLog, error=function(e) return(NA))
-    apollo_addLog("Overview of choices for MNL model component:", content, apolloLog)
+    apollo_addLog(paste0("Overview of choices for model component \"",componentName,"\""), content, apolloLog)
+    
+    apollo_reportModelTypeLog(modelType="MNL", apolloLog)
     
     return(P)
   }
