@@ -8,17 +8,17 @@
 #' @param model Model object. Estimated model object as returned by function \link{apollo_estimate}.
 #' @param apollo_probabilities Function. Returns probabilities of the model to be estimated. Must receive three arguments:
 #'                          \itemize{
-#'                            \item apollo_beta: Named numeric vector. Names and values of model parameters.
-#'                            \item apollo_inputs: List containing options of the model. See \link{apollo_validateInputs}.
-#'                            \item functionality: Character. Can be either "estimate" (default), "prediction", "validate", "conditionals", "zero_LL", or "raw".
+#'                            \item \code{apollo_beta}: Named numeric vector. Names and values of model parameters.
+#'                            \item \code{apollo_inputs}: List containing options of the model. See \link{apollo_validateInputs}.
+#'                            \item \code{functionality}: Character. Can be either "estimate" (default), "prediction", "validate", "conditionals", "zero_LL", or "raw".
 #'                          }
 #' @param apollo_inputs List grouping most common inputs. Created by function \link{apollo_validateInputs}.
 #' @param sharesTest_settings List of arguments. It must include the following.
 #'                            \itemize{
-#'                              \item alternatives: Named numeric vector. Names of alternatives and their corresponding value in \code{choiceVar}.
-#'                              \item choiceVar: Numeric vector. Contains choices for all observations. It will usually be a column from the database. Values are defined in \code{alternatives}.
-#'                              \item subsamples: Named list of boolean vectors. Each element of the list defines whether a given observation belongs to a given subsample (e.g. by sociodemographics).
-#'                              \item modelComponent: Name of model component. Set to model by default.
+#'                              \item \code{alternatives}: Named numeric vector. Names of alternatives and their corresponding value in \code{choiceVar}.
+#'                              \item \code{choiceVar}: Numeric vector. Contains choices for all observations. It will usually be a column from the database. Values are defined in \code{alternatives}.
+#'                              \item \code{subsamples}: Named list of boolean vectors. Each element of the list defines whether a given observation belongs to a given subsample (e.g. by sociodemographics).
+#'                              \item \code{modelComponent}: Name of model component. Set to model by default.
 #'                            }
 #' @return Nothing
 #' @export
@@ -33,7 +33,7 @@ apollo_sharesTest=function(model, apollo_probabilities, apollo_inputs, sharesTes
   subsamples   = sharesTest_settings[["subsamples"]]
   
   predictedShares = apollo_prediction(model, apollo_probabilities, apollo_inputs, 
-                                      sharesTest_settings$modelComponent)
+                                      prediction_settings=list(modelComponent=sharesTest_settings$modelComponent,silent=TRUE))
   
   #### predictedShares=predictedShares[,-ncol(predictedShares)] ### removed
   predictedShares=predictedShares[,!colnames(predictedShares)%in%c("ID","Choice situation","chosen")]#### NEW
@@ -80,7 +80,7 @@ apollo_sharesTest=function(model, apollo_probabilities, apollo_inputs, sharesTes
     categories[["All data"]]=rep(1,length(trueShares[[1]]))
   } else {
     if(!all(names(alternatives) %in% names(predictedShares))) stop("\nPredicted choice probabilities should be provided for all alternatives.")
-    if(any(lapply(categories,sum)==0)) stop("\nSome categories are empty!)")
+    if(any(lapply(categories,sum)==0)) stop("\nSome subsamples are empty!)")
     if(!any(lapply(categories,sum)==length(trueShares[[1]])) & !any(lapply(categories,length)==1)) categories[["All data"]]=rep(1,length(trueShares[[1]]))
   }
   
