@@ -1,11 +1,11 @@
 #' Splits data and draws for loading in cluster
 #' 
-#' Copies \code{apollo_inputs} as many times as cores, but each copy contains only part of \code{database} and \code{draws}.
+#' Splits apollo_inputs into pieces and writes them to disk (temporary folder).
 #' 
 #' Internal use only. This function is called by \link{apollo_makeCluster}.
 #' @param apollo_inputs List grouping most common inputs. Created by function \link{apollo_validateInputs}.
 #' @param silent Boolean. If TRUE, no information is printed to console or default output.
-#' @return List. Each element is a copy of apollo_inputs, but with only a piece of \code{database} and \code{draws}.
+#' @return Character vector of file names of the pieces of apollo_inputs.
 apollo_splitDataDraws <- function(apollo_inputs, silent=FALSE){
   apollo_control = apollo_inputs[["apollo_control"]]
   database       = apollo_inputs[["database"]]
@@ -24,6 +24,7 @@ apollo_splitDataDraws <- function(apollo_inputs, silent=FALSE){
   indivID  <- database[,apollo_control$indivID] # update order
   namesID  <- unique(indivID)                   # update order
   apollo_inputs$database <- database            # update order
+  rm(database); gc()
   if(!silent) cat('\nAttempting to split data into', nCores, 'pieces.',sep=' ')
   obj          <- ceiling(nObs/nCores)
   counter      <- 0
@@ -47,13 +48,11 @@ apollo_splitDataDraws <- function(apollo_inputs, silent=FALSE){
   if(!silent) cat('\n Number of observations per worker (thread):\n  ', 
                   paste0(coreLoad, collapse=", "), sep="")
   rm(obj, counter, currentCore, i, n)
-  if(!silent) cat("\n ", sum(gc()[,2]),'MB of RAM in use before splitting.', sep="")
   
   
-  ### Create list of data to be copied to each thread.
-  # Each element is a copy of apollo_inputs with fewer elements in database and draws
-  LL <- vector(mode="list", length=nCores)
-  
+  ### Create names of temp files
+  inputPieceFile <- rep("", nCores)
+  for(i in 1:nCores) inputPieceFile[i] <- tempfile()
   
   ### Identify elements to split
   # For lists, it looks at its first element
@@ -69,44 +68,37 @@ apollo_splitDataDraws <- function(apollo_inputs, silent=FALSE){
     if(nRows!=nObs & nRows!=nIndiv) asIs <- c(asIs, e)
   }; rm(e, E, nRows)
   
-  
-  ### Copy unchanged elements to each apollo_inputs
-  for(i in 1:nCores) LL[[i]] <- apollo_inputs[ asIs ]
-  rm(i)
-  
-  
-  ### Copy elements splitted by observation
-  if(length(byObs)>0) for(e in byObs){
-    if(!silent) cat('\n Splitting', e)
-    E <- apollo_inputs[[e]]
-    for(i in 1:nCores){
+  ### Create and write to disk each fragment of apollo_inputs
+  if(!silent) cat("\n Writing pieces to disk")
+  for(i in 1:nCores){
+    L <- list()
+    # Elements to be kept unchanged
+    L <- apollo_inputs[ asIs ]
+    # Elements to be splitted by observations
+    if(length(byObs)>0) for(e in byObs){
+      E <- apollo_inputs[[e]]
       r <- assignedCore==i
       if(is.list(E) & !is.data.frame(E)){
-        LL[[i]][[e]] <- lapply(E, apollo_keepRows, r=r)
-      } else {
-        LL[[i]][[e]] <- apollo_keepRows(E, r)
-      }
-      if(!silent) cat(".")
+        L[[e]] <- lapply(E, apollo_keepRows, r=r)
+      } else L[[e]] <- apollo_keepRows(E, r)
     }
-  }
-  
-  
-  ### Copy elements splitted by individual
-  if(length(byInd)>0) for(e in byInd){
-    if(!silent) cat('\n Splitting', e)
-    E <- apollo_inputs[[e]]
-    for(i in 1:nCores){
+    # Elements to be splitted by individual
+    if(length(byInd)>0) for(e in byInd){
+      E <- apollo_inputs[[e]]
       r <- namesID %in% indivID[assignedCore==i]
       if(is.list(E) & !is.data.frame(E)){
-        LL[[i]][[e]] <- lapply(E, apollo_keepRows, r=r)
-      } else {
-        LL[[i]][[e]] <- apollo_keepRows(E, r)
-      }
-      if(!silent) cat(".")
+        L[[e]] <- lapply(E, apollo_keepRows, r=r)
+      } else L[[e]] <- apollo_keepRows(E, r)
     }
+    # Write to disk
+    wroteOK <- tryCatch({
+      saveRDS(L, file=inputPieceFile[i])
+      TRUE
+    }, warning=function(w) FALSE, error=function(e) FALSE)
+    if(!wroteOK) stop("Apollo could not write data pieces to disk. This is necessary for multi-core processing to work.")
+    if(!silent) cat(".")
   }
   
-  
-  if(!silent) cat("\n Done. ", sum(gc()[,2]), 'MB of RAM in use.', sep="")
-  return(LL)
+  if(!silent) cat(" Done. ", sum(gc()[,2]), 'MB of RAM in use.', sep="")
+  return(inputPieceFile)
 }
\ No newline at end of file
